@@ -220,7 +220,6 @@ void rSetHdl(idhdl h);
 ring rInit(leftv pn, leftv rv, leftv ord);
 idhdl  rDefault(const char *s);
 
-idhdl rSimpleFindHdl(ring r, idhdl root, idhdl n=NULL);
 idhdl rFindHdl(ring r, idhdl n);
 void   rKill(idhdl h);
 void   rKill(ring r);
@@ -299,5 +298,7 @@ lists rDecompose(const ring r);
 lists rDecompose_list_cf(const ring r);
 BOOLEAN rDecompose_CF(leftv res,const coeffs C);
 ring rCompose(const lists  L, const BOOLEAN check_comp=TRUE, const long bitmask=0x7fff, const int isLetterplace=FALSE);
+
+void iiSetReturn(const leftv h);
 #endif
 
