@@ -9,18 +9,18 @@
 #ifndef COEFFS_H
 #define COEFFS_H
 
-# include <misc/auxiliary.h>
-#include <omalloc/omalloc.h>
+#include "misc/auxiliary.h"
+#include "omalloc/omalloc.h"
 
-#include <misc/sirandom.h>
+#include "misc/sirandom.h"
 /* for assume: */
-#include <reporter/reporter.h>
-#include <reporter/s_buff.h>
-#include <factory/factory.h>
+#include "reporter/reporter.h"
+#include "reporter/s_buff.h"
+#include "factory/factory.h"
 
-#include <coeffs/si_gmp.h>
-#include <coeffs/Enumerator.h>
-#include <coeffs/numstats.h> // for STATISTIC(F) counting macro
+#include "coeffs/si_gmp.h"
+#include "coeffs/Enumerator.h"
+#include "coeffs/numstats.h" // for STATISTIC(F) counting macro
 
 class CanonicalForm;
 
@@ -40,10 +40,12 @@ enum n_coeffType
                   the top-most extension in an extension tower
                   is transcendental */
   n_long_C, /**< complex floating point (GMP) numbers */
+  n_nTupel, /**< n-tupel of cf: ZZ/p1,... ZZ/pn, R, long_R */
   n_Z, /**< only used if HAVE_RINGS is defined  */
   n_Zn, /**< only used if HAVE_RINGS is defined */
   n_Znm, /**< only used if HAVE_RINGS is defined */
   n_Z2m, /**< only used if HAVE_RINGS is defined */
+  n_FlintQrat, /**< rational funtion field over Q */
   n_CF /**< ? */
 };
 
@@ -53,9 +55,8 @@ struct snumber;
 typedef struct snumber *   number;
 
 /* standard types */
-struct ip_sring;
-typedef struct ip_sring *         ring;
-typedef struct ip_sring const *   const_ring;
+//struct ip_sring;
+//typedef struct ip_sring *         ring; /* already needed in s_buff.h*/
 
 /// @class coeffs coeffs.h coeffs/coeffs.h
 ///
@@ -81,7 +82,7 @@ typedef IEnumerator<number> ICoeffsEnumerator;
 /// Additionally returns a number;
 typedef void (*nCoeffsEnumeratorFunc)(ICoeffsEnumerator& numberCollectionEnumerator, number& output, const coeffs r);
 
-extern omBin rnumber_bin;
+EXTERN_VAR omBin rnumber_bin;
 
 #define FREE_RNUMBER(x) omFreeBin((void *)x, rnumber_bin)
 #define ALLOC_RNUMBER() (number)omAllocBin(rnumber_bin)
@@ -274,8 +275,8 @@ struct n_Procs_s
    //CF: tries to find a canonical map from src -> dst
    nMapFunc (*cfSetMap)(const coeffs src, const coeffs dst);
 
-   void    (*cfWriteFd)(number a, FILE *f, const coeffs r);
-   number  (*cfReadFd)( s_buff f, const coeffs r);
+   void    (*cfWriteFd)(number a, const ssiInfo *f, const coeffs r);
+   number  (*cfReadFd)( const ssiInfo *f, const coeffs r);
 
    /// Inplace: a *= b
    void    (*cfInpMult)(number &a, number b, const coeffs r);
@@ -315,10 +316,6 @@ struct n_Procs_s
    number (*convFactoryNSingN)( const CanonicalForm n, const coeffs r);
    CanonicalForm (*convSingNFactoryN)( number n, BOOLEAN setChar, const coeffs r );
 
-
-   /// the 0 as constant, NULL by default
-   number nNULL;
-
    /// Number of Parameters in the coeffs (default 0)
    int iNumberOfParameters;
 
@@ -721,25 +718,26 @@ static FORCE_INLINE number n_Lcm(number a, number b, const coeffs r)
 static FORCE_INLINE number n_NormalizeHelper(number a, number b, const coeffs r)
 { STATISTIC(n_NormalizeHelper); assume(r != NULL); assume(r->cfNormalizeHelper!=NULL); return r->cfNormalizeHelper(a,b,r); }
 
+number ndCopyMap(number a, const coeffs src, const coeffs dst);
 /// set the mapping function pointers for translating numbers from src to dst
 static FORCE_INLINE nMapFunc n_SetMap(const coeffs src, const coeffs dst)
-{ STATISTIC(n_SetMap); assume(src != NULL && dst != NULL); assume(dst->cfSetMap!=NULL); return dst->cfSetMap(src,dst); }
+{ STATISTIC(n_SetMap);
+  assume(src != NULL && dst != NULL); assume(dst->cfSetMap!=NULL);
+  if (src==dst) return ndCopyMap;
+  return dst->cfSetMap(src,dst);
+}
 
 #ifdef LDEBUG
 /// test whether n is a correct number;
 /// only used if LDEBUG is defined
 static FORCE_INLINE BOOLEAN n_DBTest(number n, const char *filename, const int linenumber, const coeffs r)
 { STATISTIC(n_Test); assume(r != NULL); assume(r->cfDBTest != NULL); return r->cfDBTest(n, filename, linenumber, r); }
-#else
-// is it really necessary to define this function in any case?
-/// test whether n is a correct number;
-/// only used if LDEBUG is defined
-static FORCE_INLINE BOOLEAN n_DBTest(number, const char*, const int, const coeffs)
-{ STATISTIC(n_Test);  return TRUE; }
-#endif
-
 /// BOOLEAN n_Test(number a, const coeffs r)
 #define n_Test(a,r)  n_DBTest(a, __FILE__, __LINE__, r)
+#else
+#define n_Test(a,r)  1
+#endif
+
 
 /// output the coeff description
 static FORCE_INLINE void   n_CoeffWrite(const coeffs r, BOOLEAN details = TRUE)
@@ -750,22 +748,14 @@ static FORCE_INLINE void   n_CoeffWrite(const coeffs r, BOOLEAN details = TRUE)
 static FORCE_INLINE BOOLEAN nCoeff_is_Ring_2toM(const coeffs r)
 { assume(r != NULL); return (getCoeffType(r)==n_Z2m); }
 
-static FORCE_INLINE BOOLEAN nCoeff_is_Ring_ModN(const coeffs r)
-{ assume(r != NULL); return (getCoeffType(r)==n_Zn); }
-
 static FORCE_INLINE BOOLEAN nCoeff_is_Ring_PtoM(const coeffs r)
 { assume(r != NULL); return (getCoeffType(r)==n_Znm); }
 
-static FORCE_INLINE BOOLEAN nCoeff_is_Ring_Z(const coeffs r)
-{ assume(r != NULL); return (getCoeffType(r)==n_Z); }
-
 static FORCE_INLINE BOOLEAN nCoeff_is_Ring(const coeffs r)
 { assume(r != NULL); return (r->is_field==0); }
 #else
 #define nCoeff_is_Ring_2toM(A) 0
-#define nCoeff_is_Ring_ModN(A) 0
 #define nCoeff_is_Ring_PtoM(A) 0
-#define nCoeff_is_Ring_Z(A)    0
 #define nCoeff_is_Ring(A)      0
 #endif
 
@@ -838,10 +828,27 @@ static FORCE_INLINE BOOLEAN nCoeff_is_Zp(const coeffs r, int p)
 { assume(r != NULL); return ((getCoeffType(r)==n_Zp) && (r->ch == p)); }
 
 static FORCE_INLINE BOOLEAN nCoeff_is_Q(const coeffs r)
-{ assume(r != NULL); return getCoeffType(r)==n_Q && (r->is_field); }
+{
+  assume(r != NULL);
+  #if SI_INTEGER_VARIANT==1
+  return getCoeffType(r)==n_Q && (r->is_field);
+  #else
+  return getCoeffType(r)==n_Q;
+  #endif
+}
 
 static FORCE_INLINE BOOLEAN nCoeff_is_Z(const coeffs r)
-{ assume(r != NULL); return getCoeffType(r)==n_Z || ((getCoeffType(r)==n_Q) && (!r->is_field)); }
+{
+  assume(r != NULL);
+  #if SI_INTEGER_VARIANT==1
+  return ((getCoeffType(r)==n_Q) && (!r->is_field));
+  #else
+  return getCoeffType(r)==n_Z;
+  #endif
+}
+
+static FORCE_INLINE BOOLEAN nCoeff_is_Zn(const coeffs r)
+{ assume(r != NULL); return getCoeffType(r)==n_Zn; }
 
 static FORCE_INLINE BOOLEAN nCoeff_is_Q_or_BI(const coeffs r)
 { assume(r != NULL); return getCoeffType(r)==n_Q; }
@@ -984,11 +991,11 @@ static FORCE_INLINE number n_Random(siRandProc p, number p1, number p2, const co
 { STATISTIC(n_Random); assume( cf != NULL ); assume( cf->cfRandom != NULL );  return cf->cfRandom(p, p1, p2, cf); }
 
 /// io via ssi:
-static FORCE_INLINE void n_WriteFd(number a, FILE *f, const coeffs r)
+static FORCE_INLINE void n_WriteFd(number a, const ssiInfo *f, const coeffs r)
 { STATISTIC(n_WriteFd); assume(r != NULL); assume(r->cfWriteFd != NULL); return r->cfWriteFd(a, f, r); }
 
 /// io via ssi:
-static FORCE_INLINE number n_ReadFd( s_buff f, const coeffs r)
+static FORCE_INLINE number n_ReadFd( const ssiInfo *f, const coeffs r)
 { STATISTIC(n_ReadFd); assume(r != NULL); assume(r->cfReadFd != NULL); return r->cfReadFd(f, r); }
 
 
